# vim: set fileencoding=utf-8 :

from katrin.models import *
from django.contrib import admin
from django import forms
from django.forms.util import ErrorList
from django.forms.models import BaseInlineFormSet


from django.contrib.auth.models import User
from django.contrib.admin import widgets
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import string_concat

class MyServiceAdminForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		super(MyServiceAdminForm, self).__init__(*args, **kwargs)
		instance = getattr(self, 'instance', None)
		if instance and instance.id:
			self.fields['codename'].widget.attrs['readonly'] = True

class ServiceAdmin(admin.ModelAdmin):
	list_display = ('title', 'codename',)
	form = MyServiceAdminForm


def user_service_form_factory(user):
	class RuntimeUserServiceForm(forms.ModelForm):
		def __init__(self, *args, **kwargs):
			if self.base_fields.has_key('account'):
				if isinstance(user, Client):
					self.base_fields['account'].queryset = Account.objects.filter(client=user)
				if isinstance(user, ClientGroup):
					self.base_fields['account'].queryset = Account.objects.filter(group=user)
			super(RuntimeUserServiceForm, self).__init__(*args, **kwargs)
 
		def clean(self):
			cleaned_data = self.cleaned_data
			tariff = self.cleaned_data['tariff']
			service = self.cleaned_data['service']
			if not tariff.has_this_service(service):
				self._errors['tariff'] = ErrorList([_("Chosen tariff isn't for chosen service")])
				del cleaned_data['tariff']
				del cleaned_data['service']

			return cleaned_data

		class Meta:
			model = UserService

	return RuntimeUserServiceForm

class UserServiceInlineFormset(BaseInlineFormSet):

	def clean(self):
		""" Checks if two userservices have the same service """
		if any(self.errors):
			# Don't bother validating the formset unless each form is valid on its own
			return
		services = []
		for i in range(0, self.total_form_count()):
			form = self.forms[i]
			if form.cleaned_data.has_key('DELETE') and not form.cleaned_data['DELETE']:
				if form.cleaned_data.has_key('service'):
					service = form.cleaned_data['service']
					if service in services:
						raise forms.ValidationError, _("Services in a set must be unique.")
					services.append(service)



class UserServiceAdminInline(admin.TabularInline):

	model = UserService

	formset = UserServiceInlineFormset

	def get_formset(self, request, obj=None, **kwargs):
		self.fields = ('service','tariff')
		print obj
		if obj is not None:
			self.fields = ('service','tariff','account')
			self.form = user_service_form_factory(obj) # obj is a Client or ClientGroup
		return super(UserServiceAdminInline, self).get_formset(request, obj, **kwargs)

class ClientAdmin(admin.ModelAdmin):
	fieldsets = (
		(_('General info'), {'fields': ('login','password','lastname','firstname','middlename',)}),
		(_('Contacts'), {'fields': ('work_tel', 'mobile_tel', 'email', 'jabber')}),
		(_('Legal info'), {'fields': ('yu_name', 'yu_address')}),
	)
	list_display = ('login', 'lastname', 'yu_name','number_of_accounts')
	ordering = ['login']
	search_fields = ('login', 'lastname')
	inlines = [
			UserServiceAdminInline,
			]

	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)

class ClientGroupAdmin(admin.ModelAdmin):
	fieldsets = (
		(None, {'fields': ('name',)}),
		(_('Members'), {'fields': ('clients',)}),
		(_('Legal info'), {'fields': ('yu_name', 'yu_address')}),
	)
	list_display = ('name','yu_name','number_of_accounts')
	search_fields = ('name',)
	filter_horizontal = ('clients',)
	inlines = [
			UserServiceAdminInline,
			]
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)


class MyAccountAdminForm(forms.ModelForm):
	class Media:
		css = {
			'all': ('/katrin/adminmedia/account.css',)
			}
		js = ('/katrin/adminmedia/account.js',)
	def __init__(self, *args, **kwargs):
		super(MyAccountAdminForm, self).__init__(*args, **kwargs)
		self.fields['balance'].widget.attrs['readonly'] = True
	

	class Meta:
		model = Account
	def clean(self):
		cleaned_data = self.cleaned_data
		client = cleaned_data['client']
		group = cleaned_data['group']
		type = cleaned_data['type']

		if type == AccountType['Personal']:
			if client == None:
				self._errors['client'] = ErrorList([_('You have to choose client')])
				del cleaned_data['client']
		elif type == AccountType['Group']:
			if group == None:
				self._errors['group'] = ErrorList([_('You have to choose group')])
				del cleaned_data['group']
		else:
			raise forms.ValidationError(u'%s: %s' % (_('Unknown type'), type))

		return cleaned_data


class AccountAdmin(admin.ModelAdmin):

	fieldsets = (
		(None,{'fields': ('name','type','postpaid')}),
		(_('Owner'), {
			'fields': ('client','group','ext_owner_id'),
			'classes': ('owner',),
			}),
		(_('Account state'), { 'fields': ('balance','credit','blocked') }),

	)
	list_display = ('name','blocked','balance','credit','get_owner')
	search_fields = ('name',)
	list_filter = ('blocked',)
	radio_fields = {'blocked': admin.VERTICAL, 'type': admin.VERTICAL}
	actions = ['close_billing_period']
	form = MyAccountAdminForm

	def close_billing_period(self, request, queryset):
		rows_updated = queryset.count()
		for obj in queryset:
			obj.close_billing_period(obj.userservices.all(), True)
		message_bit = u"%s %s" % (_("Number of processed objects is"), rows_updated)
		self.message_user(request, message_bit)

	close_billing_period.short_description = _('Close billing period for all services of selected accounts')

	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)

class AccountHistoryAdmin(admin.ModelAdmin):
	date_hierarchy = 'date_time'
	list_display = ('date_time', 'account', 'get_admin', 'balance', 'credit','blocked', 'get_operation')
	ordering = ['-date_time', 'account']
	list_filter = ('admin', 'date_time',)
	def save_model(self, request, obj, form, change):
		obj.admin = request.user
		obj.save()

class MyOperationAdminForm(forms.ModelForm):
	class Media:
		css = {
			'all': ('/katrin/adminmedia/operation.css',)
			}
		js = ('/katrin/adminmedia/operation.js',)
	class Meta:
		model = Operation
	def clean_debit(self):
		debit = self.cleaned_data['debit']
		if debit < 0:
			raise forms.ValidationError(_("Debit can't be negative"))
		return debit

	def clean_credit(self):
		credit = self.cleaned_data['credit']
		if credit < 0:
			raise forms.ValidationError(_("Credit can't be negative"))
		return credit

	def clean_service(self):
		service = self.cleaned_data['service']
		type = self.cleaned_data['type']
		if type in (
				OperationType['Activation'], 
				OperationType['Periodic'],
				OperationType['Refund'], 
				OperationType['Usage'],
				OperationType['Closure'],
				) and not service:
			raise forms.ValidationError(_("This type of operation requires to specify service"))
		return service

class OperationAdmin(admin.ModelAdmin):
	fieldsets = (
		(None, {'fields': ('description', 'account', 'type', 'service', 'date_time', 'status')}),
		(_('Operation amount'), {'fields': ('debit', 'credit',),
				'classes': ('amount',)}),
	)
	list_display = ('date_time', 'account', 'description', 'type', 'get_service', 'debit', 'credit','status')
	search_fields = ('description',)
	list_filter = ('type', 'service', 'status')
	date_hierarchy = 'date_time'
	form = MyOperationAdminForm
	actions = ['set_status_no_bill','set_status_closed']

	def set_status_no_bill(self, request, queryset):
		rows_updated = queryset.update(status=OperationStatus['No bill'])
		message_bit = u"%s %s" % (_("Number of processed objects is"), rows_updated)
		self.message_user(request, message_bit)
	set_status_no_bill.short_description = _('Set status "No bill"')

	def set_status_closed(self, request, queryset):
		rows_updated = queryset.update(status=OperationStatus['Closed'])
		message_bit = u"%s %s" % (_("Number of processed objects is"), rows_updated)
		self.message_user(request, message_bit)
	set_status_closed.short_description = _('Set status "Closed"')

class TariffAdmin(admin.ModelAdmin):
	fieldsets = (
		(_('Tariff info'), {'fields': ('title', 'service', 'billing_period')}),
		(_('Tariff details'), {'fields': ('activation_fee', 'periodic_fee', 'week_days')}),
	)
	list_display = ('title', 'service', 'billing_period', 'activation_fee', 'periodic_fee', 'week_days')
	ordering = ('service', 'title')
	search_fields = ('title',)
	list_filter = ('service','billing_period')

class BillEntryAdmin(admin.TabularInline):
	model = BillEntry
	fields = ('operation', 'net_amount', 'discount', 'vat_amount', )

class BillAdmin(admin.ModelAdmin):
	fieldsets = (
		(None, {'fields': ('bill_to', 'bill_to_extid','date',)}),
	)
	date_hierarchy = 'date'
	list_display = ('date', 'bill_to','bill_to_extid','total')
	inlines = [
			BillEntryAdmin,
			]

class BillingPeriodAdmin(admin.ModelAdmin):
	pass

class RefillLogAdmin(admin.ModelAdmin):
	date_hierarchy = 'datetime'
	list_display = ('datetime', 'admin','account','refill_size')

admin.site.register(Service, ServiceAdmin)
admin.site.register(FilterGroup)
admin.site.register(Client, ClientAdmin)
admin.site.register(ClientGroup, ClientGroupAdmin)
admin.site.register(Account, AccountAdmin)
admin.site.register(AccountHistory, AccountHistoryAdmin)
admin.site.register(Operation, OperationAdmin)
admin.site.register(Bill, BillAdmin)
admin.site.register(BillingPeriod, BillingPeriodAdmin)
admin.site.register(Tariff, TariffAdmin)
#admin.site.register(RefillLog, RefillLogAdmin)
#admin.site.register(Refill)
