# vim: set fileencoding=utf-8 :

from django.db import models
from django.db.models import Sum, Count
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from datetime import datetime
from decimal import Decimal

from katrin.models import *

class ServiceTariff(models.Model):
	id = models.AutoField(primary_key=True)
	title = models.CharField(u'Название', max_length=40)
	tariff = models.OneToOneField(Tariff, verbose_name=u'Основной тариф')
	def __unicode__(self):
		return "%s" % (self.title)
	class Meta:
		abstract = True

class ServiceFilter(models.Model):
	"""Title is needed for usage operation description. If filter is a member of filter group then
	group name is used for description.
	"""
	id = models.AutoField(primary_key=True)
	title = models.CharField(_('Name'), max_length=80)
	tariff = models.ForeignKey(Tariff, verbose_name=_('Tariff'))
	filter_group = models.ForeignKey(FilterGroup, blank=True, null=True, verbose_name=_('Filter group'))
	priority = models.IntegerField(_('Priority'), default=0)
	store_stat = models.IntegerField(_('Save statistics'), default=1, choices=YES_NO)
	def __unicode__(self):
		return u"%s" % (self.title)
	class Meta:
		abstract = True

class ServiceStat(models.Model):
	id = models.AutoField(primary_key=True)
	client = models.ForeignKey(Client, verbose_name=_('Client'))
	date_time = models.DateTimeField(_('Date and time'), db_index=True)
	#filter = create foreign key to your non abstract ServiceFilter in your ServiceStat
	cost = models.FloatField(_('cost'))

	class Meta:
		abstract = True
	
	@classmethod
	def create_usage_operations(cls, userservice, include_last_stat=False):
		"""include_last_stat - whether to count stat in last aggregating period which hasn't ended"""
		#print cls
		start_time = cls.get_start_time(userservice)
		if not start_time:
			# No usage
			return
		end_time = cls.get_end_time(include_last_stat)
		#print "start %s, end %s" % (start_time, end_time)
		if userservice.client:
			cls.create_client_usage_operations(userservice, start_time, end_time)
		if userservice.group:
			cls.create_clientgroup_usage_operations(userservice, start_time, end_time)

	@classmethod
	def create_client_usage_operations(cls, clientservice, start_time, end_time):
		if start_time >= end_time:
			raise ValueError("Start time should be earlier then end time")
		client=clientservice.client
		filter_queryset = cls.objects.filter(client=client, date_time__range=(start_time,end_time)).values('client','filter','filter__title', 'filter__filter_group',).annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=True)
		filtergroup_queryset = cls.objects.filter(client=client, date_time__range=(start_time,end_time)).values('client','filter__filter_group','filter__filter_group__name').annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=False)
		cls.querysets_to_operations(clientservice, filter_queryset, filtergroup_queryset)

	@classmethod
	def create_clientgroup_usage_operations(cls, clientgroupservice, start_time, end_time):
		if start_time >= end_time:
			raise ValueError("Start time should be earlier then end time")
		clientgroup=clientgroupservice.group
		filter_queryset = cls.objects.filter(client__in=clientgroup.members_ids, date_time__range=(start_time,end_time)).values('filter','filter__title', 'filter__filter_group',).annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=True)
		filtergroup_queryset = cls.objects.filter(client__in=clientgroup.members_ids, date_time__range=(start_time,end_time)).values('filter__filter_group','filter__filter_group__name').annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=False)
		cls.querysets_to_operations(clientgroupservice, filter_queryset, filtergroup_queryset)

	@classmethod
	def querysets_to_operations(cls, userservice, filter_queryset, filtergroup_queryset):
		print filter_queryset
		for item in filtergroup_queryset.all():
			debit = Decimal('%f' % item['cost_sum'])
			descr = item['filter__filter_group__name']
			Operation.do_usage(userservice, descr, debit)
		print filtergroup_queryset
		for item in filter_queryset.all():
			debit = Decimal('%f' % item['cost_sum'])
			descr = item['filter__title']
			Operation.do_usage(userservice, descr, debit)

	@classmethod
	def get_start_time(cls, userservice):
		closure = userservice.get_previous_period_closure()
		if not closure:
			print "closure is null"
			# There is no closure yet, try to get first stat for given userservice
			if userservice.client:
				start_stat = cls.get_first_client_stat(userservice.client)
			else:
				start_stat = cls.get_first_clientgroup_stat(userservice.group)
			if not start_stat:
				# Service hasn't been used yet
				print "no service stat"
				return None
			else:
				print "start_stat %s, %s" % (start_stat.client, start_stat.date_time)
				return start_stat.date_time
		else:
			print "closure was found: %s" % closure
			return cls.apply_stat_shift(closure.date_time)

	@classmethod
	def get_first_client_stat(cls, client):
		if not client:
			raise ValueError("Specify client to get his first stat")

		res = cls.objects.filter(client=client).order_by('date_time')[:1]
		if len(res) == 1:
			 return res[0]
		else:
			 return None

	@classmethod
	def get_first_clientgroup_stat(cls, clientgroup):
		if not clientgroup:
			raise ValueError("Specify clientgroup to get it's first stat")
		first_stats = []
		for client in clientgroup.clients.all():
			first_stat = cls.get_first_client_stat(client)
			if first_stat:
				first_stats.append(first_stat)
		if len(first_stats) > 0:
			return min(first_stats)
		else:
			return None

	@classmethod
	def get_end_time(cls, include_last_stat=False):
		if include_last_stat:
			end_time=datetime.now()
		else:
			end_time=cls.apply_stat_shift(datetime.now())
		return end_time
	
	@classmethod
	def apply_stat_shift(cls, date_time):
		"""No shift by default"""
		return date_time

	@staticmethod
	def aggregate_stat(service_codename, level, entry_time):
		stat_class = ServiceStat.get_stat_class(service_codename, level)
		if stat_class:
			stat_class.aggregate(entry_time)
		else:
			print "Can't find stat class for %s service" % service_codename

	@staticmethod
	def clean_stat(service_codename, start_time, end_time):
		stat_class = ServiceStat.get_stat_class(service_codename)
		if stat_class:
			stat_class.clean(start_time, end_time)

	@classmethod
	def aggregate(cls, entry_time):
		start_time = entry_time
		end_time = cls.apply_stat_shift(entry_time, 1)
		queryset = cls.aggregation_source().objects.filter(date_time__range=(start_time,end_time)).\
				values('filter','client',).annotate(cost_sum=Sum('cost'))
		for item in queryset:
			cls(
				date_time=entry_time,
				filter_id=item['filter'],
				client_id=item['client'],
				cost=item['cost_sum'],
				).save()

	@classmethod
	def clean(cls, start_time, end_time):
		cls.objects.filter(date_time__range=(start_time,end_time)).delete()
	
	@classmethod
	def create_usage_operations(cls, userservice, include_last_stat=False):
		"""include_last_stat - whether to count stat in last aggregating period which hasn't ended"""
		#print cls
		start_time = cls.get_start_time(userservice)
		if not start_time:
			# No usage
			return
		end_time = cls.get_end_time(include_last_stat)
		#print "start %s, end %s" % (start_time, end_time)
		if userservice.client:
			cls.create_client_usage_operations(userservice, start_time, end_time)
		if userservice.group:
			cls.create_clientgroup_usage_operations(userservice, start_time, end_time)

	@classmethod
	def create_client_usage_operations(cls, clientservice, start_time, end_time):
		if start_time >= end_time:
			raise ValueError("Start time should be earlier then end time")
		client=clientservice.client
		filter_queryset = cls.objects.filter(client=client, date_time__range=(start_time,end_time)).values('client','filter','filter__title', 'filter__filter_group',).annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=True)
		filtergroup_queryset = cls.objects.filter(client=client, date_time__range=(start_time,end_time)).values('client','filter__filter_group','filter__filter_group__name').annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=False)
		cls.querysets_to_operations(clientservice, filter_queryset, filtergroup_queryset)

	@classmethod
	def create_clientgroup_usage_operations(cls, clientgroupservice, start_time, end_time):
		if start_time >= end_time:
			raise ValueError("Start time should be earlier then end time")
		clientgroup=clientgroupservice.group
		filter_queryset = cls.objects.filter(client__in=clientgroup.members_ids, date_time__range=(start_time,end_time)).values('filter','filter__title', 'filter__filter_group',).annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=True)
		filtergroup_queryset = cls.objects.filter(client__in=clientgroup.members_ids, date_time__range=(start_time,end_time)).values('filter__filter_group','filter__filter_group__name').annotate(cost_sum=Sum('cost')).filter(filter__filter_group__isnull=False)
		cls.querysets_to_operations(clientgroupservice, filter_queryset, filtergroup_queryset)

	@classmethod
	def querysets_to_operations(cls, userservice, filter_queryset, filtergroup_queryset):
		for item in filtergroup_queryset.all():
			debit = Decimal('%f' % item['cost_sum'])
			descr = item['filter__filter_group__name']
			Operation.do_usage(userservice, descr, debit)
		for item in filter_queryset.all():
			debit = Decimal('%f' % item['cost_sum'])
			descr = item['filter__title']
			Operation.do_usage(userservice, descr, debit)

	@classmethod
	def get_start_time(cls, userservice):
		closure = userservice.get_previous_period_closure()
		if not closure:
			#print "closure is null"
			# There is no closure yet, try to get first stat for given userservice
			if userservice.client:
				start_stat = cls.get_first_client_stat(userservice.client)
			else:
				start_stat = cls.get_first_clientgroup_stat(userservice.group)
			if not start_stat:
				# Service hasn't been used yet
				#print "no service stat"
				return None
			else:
				#print "start_stat %s, %s" % (start_stat.client, start_stat.date_time)
				return start_stat.date_time
		else:
			#print "closure was found: %s" % closure
			return cls.apply_stat_shift(closure.date_time)

	@classmethod
	def get_first_client_stat(cls, client):
		if not client:
			raise ValueError("Specify client to get his first stat")

		res = cls.objects.filter(client=client).order_by('date_time')[:1]
		if len(res) == 1:
			 return res[0]
		else:
			 return None

	@classmethod
	def get_first_clientgroup_stat(cls, clientgroup):
		if not clientgroup:
			raise ValueError("Specify clientgroup to get it's first stat")
		first_stats = []
		for client in clientgroup.clients.all():
			first_stat = cls.get_first_client_stat(client)
			if first_stat:
				first_stats.append(first_stat)
		if len(first_stats) > 0:
			return min(first_stats)
		else:
			return None

	@classmethod
	def get_end_time(cls, include_last_stat=False):
		if include_last_stat:
			end_time=datetime.now()
		else:
			end_time=cls.apply_stat_shift(datetime.now())
		return end_time
	
	@classmethod
	def apply_stat_shift(cls, date_time):
		"""No shift by default"""
		return date_time

	@classmethod
	def aggregate(cls, entry_time):
		print "Override aggregate in your stat class"

	@classmethod
	def get_stat_class_name(cls, service_codename, agg_level=''):
		''' agg_level - see PeriodDevisibles'''
		return "Service%sStat%s" % (service_codename.capitalize(), agg_level)

	@classmethod
	def get_stat_class(cls, service_codename, agg_level=''):
		if service_codename in settings.KATRIN_MODULES:
			try:
				module_name = "%s.models" % service_codename
				class_name = "%s" % cls.get_stat_class_name(service_codename, agg_level)
				module = __import__(module_name, globals(), locals(), [class_name])
				if not hasattr(module, class_name):
					print ("Module %s doesn't have class %s. Can't get stat class." % (module_name, class_name))
				else:
					return getattr(module, class_name)
			except(ImportError):
				raise ImportError("Can't load katrin app %s." % service_codename)

class ServiceUparam(models.Model):
	id = models.AutoField(primary_key=True)
	client = models.ForeignKey(Client, verbose_name=_('Client'))
	class Meta:
		abstract = True
	def __unicode__(self):
		return u"%s" % self.client 

